@@ -14,6 +14,7 @@ from tempfile import mkstemp
 from ofs.base import OFSInterface, OFSException
 import boto
 import boto.exception
+import boto.connection
 
 class BotoOFS(OFSInterface):
     '''s3 backend for OFS.
@@ -25,38 +26,32 @@ class BotoOFS(OFSInterface):
         self.conn = conn
         self._bucket_cache = {}
     
-    
     def _get_bucket(self, bucket_name):
         if not bucket_name in self._bucket_cache.keys():
             self._bucket_cache[bucket_name] = self.conn.lookup(bucket_name)
         return self._bucket_cache[bucket_name]
     
-    
     def _require_bucket(self, bucket_name):
         """ Also try to create the bucket. """
         if not self.exists(bucket_name) and not self.claim_bucket(bucket_name):
             raise OFSException("Invalid bucket: %s" % bucket_name)
         return self._get_bucket(bucket_name)
-    
         
     def _get_key(self, bucket, label):
         return bucket.get_key(label)
     
-    
     def _require_key(self, bucket, label):
         key = self._get_key(bucket, label)
         if key is None:
             raise OFSException("%s->%s does not exist!" % (bucket.name, label))
         return key
     
-    
     def exists(self, bucket, label=None):
         bucket = self._get_bucket(bucket)
         if bucket is None: 
             return False
         return (label is None) or (label in bucket)
     
-    
     def claim_bucket(self, bucket):
         try:
             if self.exists(bucket):
@@ -66,7 +61,6 @@ class BotoOFS(OFSInterface):
         except boto.exception.S3CreateError, sce:
             return False
     
-    
     def _del_bucket(self, bucket):
         if self.exists(bucket):
             bucket = self._get_bucket(bucket)
@@ -75,18 +69,15 @@ class BotoOFS(OFSInterface):
             bucket.delete()
             del self._bucket_cache[bucket.name]
     
-    
     def list_labels(self, bucket):
         _bucket = self._get_bucket(bucket)
         for key in _bucket.list():
             yield key.name
 
-        
     def list_buckets(self):
         for bucket in self.conn.get_all_buckets():
             self._bucket_cache[bucket.name] = bucket
             yield bucket.name
-    
 
     def get_stream(self, bucket, label, as_stream=True):
         bucket = self._require_bucket(bucket)
@@ -95,14 +86,12 @@ class BotoOFS(OFSInterface):
             return key.get_contents_as_string()
         return key
     
-    
     def get_url(self, bucket, label):
         bucket = self._require_bucket(bucket)
         key = self._require_key(bucket, label)
         key.make_public()
         # expire can be negative when data is public
         return key.generate_url(-1) 
-    
 
     def put_stream(self, bucket, label, stream_object, params={}):
         bucket = self._require_bucket(bucket)
@@ -118,7 +107,6 @@ class BotoOFS(OFSInterface):
         self._update_key_metadata(key, params)
         key.set_contents_from_file(stream_object)
         key.close()
-    
 
     def del_stream(self, bucket, label):
         """ Will fail if the bucket or label don't exist """
@@ -126,13 +114,11 @@ class BotoOFS(OFSInterface):
         key = self._require_key(bucket, label)
         key.delete()
 
-
     def get_metadata(self, bucket, label):
         bucket = self._require_bucket(bucket)
         key = self._require_key(bucket, label)
         
-        # HACK: Amazon keeps getting hiccups when we actually use their metadata fields. 
-        meta = json.loads(key.metadata.get('_meta', '{}'))
+        meta = dict(key.metadata)
         meta.update({
             '_bucket': bucket.name,
             '_label': label,
@@ -143,7 +129,6 @@ class BotoOFS(OFSInterface):
         })
         return meta
     
-    
     def _update_key_metadata(self, key, params):
         if '_format' in params:
             key.content_type = params['_format']
@@ -152,30 +137,18 @@ class BotoOFS(OFSInterface):
         if '_owner' in params:
             key.owner = params['_owner']
             del params['_owner']
-        
-        # HACK: Amazon keeps getting hiccups when we actually use their metadata fields. 
-        key.update_metadata({'_meta': json.dumps(params)})
-        
+        for name in ['_label', '_bucket', '_last_modified', '_content_length']:
+            if name in params:
+                del params[name]
+        key.update_metadata(params)
 
     def update_metadata(self, bucket, label, params):
         key = self._require_key(self._require_bucket(bucket), label)
         self._update_key_metadata(key, params)
-        
-        #### HACK: Boto will not submit new metadata unless the file 
-        #### is uploaded again. 
-        (fp, fn) = mkstemp()
-        f_in = open(fn, 'wb')
-        key.get_contents_to_file(f_in)
-        f_in.close()
-        f_out = open(fn, 'rb')
-        key.set_contents_from_file(f_out)
-        f_out.close()
-        os.close(fp)
-        #### END HACK
-        
+        # cannot update metadata on its own. way round this is to copy file
+        key.copy(key.bucket, key.name, dict(key.metadata), preserve_acl=True)
         key.close()
     
-
     def del_metadata_keys(self, bucket, label, keys):
         key = self._require_key(self._require_bucket(bucket), label)
         for _key, value in key.metadata.items():
@@ -183,6 +156,36 @@ class BotoOFS(OFSInterface):
                 del key.metadata[_key] 
         key.close()
 
+    def authenticate_request(self, method, bucket='', key='', headers=None):
+        '''Authenticate a HTTP request by filling in Authorization field header.
+
+        :param method: HTTP method (e.g. GET, PUT, POST)
+        :param bucket: name of the bucket.
+        :param key: name of key within bucket.
+        :param headers: dictionary of additional HTTP headers.
+
+        :return: boto.connection.HTTPRequest object with Authorization header
+        filled (NB: will also have a Date field if none before and a User-Agent
+        field will be set to Boto).
+        '''
+        # following is extracted from S3Connection.make_request and the method
+        # it calls: AWSAuthConnection.make_request
+        path = self.conn.calling_format.build_path_base(bucket, key)
+        auth_path = self.conn.calling_format.build_auth_path(bucket, key)
+        http_request = boto.connection.AWSAuthConnection.build_base_http_request(
+                self.conn,
+                method,
+                path,
+                auth_path,
+                {},
+                headers
+                )
+        http_request = boto.connection.AWSAuthConnection.fill_in_auth(
+                self.conn,
+                http_request)
+        return http_request
+
+
 class S3OFS(BotoOFS):
     
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
@@ -191,6 +194,7 @@ class S3OFS(BotoOFS):
         conn = boto.connect_s3(aws_access_key_id, aws_secret_access_key, **kwargs)
         super(S3OFS, self).__init__(conn)
 
+
 class GSOFS(BotoOFS):
 
     def __init__(self, gs_access_key_id=None, gs_secret_access_key=None, **kwargs):
